using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Sockets;
using System.ComponentModel;

namespace SequenceSynchroTrigger
{
    public class DataClient
    {
        /// <summary>
        /// Connection with the system
        /// </summary>
        private Socket socket;

        /// <summary>
        /// The ip of the system
        /// </summary>
        private String systemIp;

        /// <summary>
        /// The port that we listen to
        /// </summary>
        private int systemPort;

        /// <summary>
        /// The size of one packet
        /// </summary>
        private int size;

        /// <summary>
        /// Background worker that enables to handle the socket reception
        /// </summary>
        private BackgroundWorker worker;

        /// <summary>
        /// Delegate to become event when a packet is received
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public delegate void PacketReceivedEventHandler(object sender, DataClientEventArgs e);

        /// <summary>
        /// Definition of the evenement
        /// </summary>
        public event PacketReceivedEventHandler packetReceived;

        private List<DataClientEventArgs> receivedPackets;

        public int receptedCount = 0;

        private static object syncLock = new object();

        private int connectionState = 0;

        /// <summary>
        /// Constructor of the class
        /// </summary>
        /// <param name="ipAddress">The ip address of the system</param>
        /// <param name="port">The port used</param>
        /// <param name="packetSize">The size of one packet to receive in byte</param>
        public DataClient(String ipAddress, int port, int packetSize)
        {
            socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            this.receivedPackets = new List<DataClientEventArgs>();

            // Copy params
            this.systemIp = ipAddress;
            this.systemPort = port;
            this.size = packetSize;

            // Create the background worker
            worker = new BackgroundWorker();
            worker.WorkerSupportsCancellation = true;
            worker.WorkerReportsProgress = true;
            worker.DoWork += new DoWorkEventHandler(worker_DoWork);
        }

        /// <summary>
        /// Work of the background worker
        /// Receive data from the socket
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void worker_DoWork(object sender, DoWorkEventArgs e)
        {
            // Allocate space
            byte[] buffer = new byte[32768];

            DataClientEventArgs eventArgs = new DataClientEventArgs();
            eventArgs.data = new byte[this.size];
            int eventBufferFreeSize = this.size;
            int eventBufferIndex = 0;

            try
            {
                while (this.worker.CancellationPending == false)
                {
                    // Receive a large chunk
                    // Receiving a large chunk to then convert it is faster than receiving small chunk of data.
                    int recepted = socket.Receive(buffer);

                    receptedCount += recepted;

                    int offset = 0;
                    while (recepted > 0)
                    {
                        int chunk = (recepted > eventBufferFreeSize) ? eventBufferFreeSize : recepted;
                        // Copy buffer
                        Buffer.BlockCopy(buffer, offset, eventArgs.data, eventBufferIndex, chunk);
                        // Update
                        recepted -= chunk;
                        offset += chunk;
                        eventBufferIndex += chunk;
                        eventBufferFreeSize -= chunk;
                        // Do we have to send ?
                        if (eventBufferFreeSize == 0)
                        {
                            eventArgs.receptionTime = DateTime.Now;

                            OnPacketReceived(eventArgs);

                            // Reset
                            eventBufferFreeSize = this.size;
                            eventBufferIndex = 0;

                            // Store it
                            lock (syncLock)
                            {
                                this.receivedPackets.Add(eventArgs);
                            }

                            // New arg
                            eventArgs = new DataClientEventArgs();
                            eventArgs.data = new byte[this.size];
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                this.connectionState = 0;
                Console.WriteLine("Error : " + ex.Message);
            }
        }

        /// <summary>
        /// Signal the event to the listeners
        /// </summary>
        /// <param name="e"></param>
        protected virtual void OnPacketReceived(DataClientEventArgs e)
        {
            if (packetReceived != null)
            {
                packetReceived(this, e);
            }
        }

        /// <summary>
        /// Start to listen from the packet
        /// </summary>
        /// <returns></returns>
        public int StartListening()
        {
            // Flush the reception list
            this.receivedPackets.Clear();

            // Try to connect
            try
            {
                socket.Connect(systemIp, systemPort);
            }
            catch (Exception ex)
            {
                Console.WriteLine("socket connection exception" + ex.Message);
                return -1;
            }

            this.connectionState = 1;

            // Since it may takes some time to establish the connection, we add a sleep here
            System.Threading.Thread.Sleep(200);

            worker.RunWorkerAsync();
            return 0;
        }

        /// <summary>
        /// Close connection
        /// </summary>
        public void Close()
        {
            // Close the connection
            try
            {
                socket.Close();
            }
            catch (Exception) { };

            // Stop the worker
            worker.CancelAsync();
            worker.Dispose();
        }

        /// <summary>
        /// Print available bytes in the socket
        /// </summary>
        public void PrintAvailable()
        {
            try
            {
                Console.WriteLine("Available : " + socket.Available);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception : " + ex.Message);
            }
        }

        /// <summary>
        /// Pop the first packet
        /// </summary>
        /// <returns></returns>
        public DataClientEventArgs PopPacket()
        {
            DataClientEventArgs r = null;

            if (this.receivedPackets.Count <= 0)
                return null;

            lock (syncLock)
            {
                r = this.receivedPackets[0];
                this.receivedPackets.RemoveAt(0);
            }
            return r;
        }

        /// <summary>
        /// Pop the last packet available and flush the rest
        /// </summary>
        /// <param name="flushedCount"></param>
        /// <returns></returns>
        public DataClientEventArgs PopAndFlush(ref int flushedCount)
        {
            DataClientEventArgs r = null;

            if (this.receivedPackets.Count <= 0)
                return null;

            lock (syncLock)
            {
                r = this.receivedPackets[this.receivedPackets.Count-1];
                flushedCount = this.receivedPackets.Count;
                this.receivedPackets.Clear();
            }
            return r;
        }

        /// <summary>
        /// Get a packet by index
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public DataClientEventArgs GetPacket(int index)
        {
            return this.receivedPackets[index];
        }

        /// <summary>
        /// Number of packet in the fifo
        /// </summary>
        /// <returns></returns>
        public int GetNbrOfPacketReceived()
        {
            int p = 0;
            lock (syncLock)
            {
                p = this.receivedPackets.Count;
            }
            return p;
        }

        public int ConnectionState
        {
            get { return this.connectionState; }
        }
    }
}
