using System;
using System.Collections.Generic;
using System.Text;

namespace ReadDigitalInputs
{
    public class Msxe
    {
        /// <summary>
        /// Common web reference
        /// </summary>
        private MXCommonWeb.MXCommon commonWeb;

        /// <summary>
        /// Store the connection state of the system
        /// 0: not connected
        /// 1: connected
        /// </summary>
        protected int connectionState;

        /// <summary>
        /// Store the IP address of the system
        /// </summary>
        private string ip;

        /// <summary>
        /// Store the type of the system. It is in fact the result of the GetModuleType function
        /// Will be set in the Connect method
        /// </summary>
        private string systemType;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="ip">IP address of the system</param>
        public Msxe(string ip)
        {
            // Store the parameters
            this.ip = ip;

            // Default
            this.connectionState = 0;
        }

        /// <summary>
        /// Open a connection with the system
        /// </summary>
        /// <param name="error"></param>
        /// <returns>
        /// 0:  success
        /// -1: wrong IP address
        /// -2: system error. Check the error variable for more informations
        /// -3: wrong system type
        /// -4: system not reachable
        /// -5: other error. Check the error variable for more informations
        /// </returns>
        public virtual int Connect(out string error)
        {
            int retval = 0;

            // Default
            error = string.Empty;

            // Reset the connection state
            this.connectionState = 0;

            // Create the web-reference object
            this.commonWeb = new MXCommonWeb.MXCommon();
            try
            {
                this.commonWeb.Url = "http://" + this.ip + ":5555";
                this.commonWeb.Timeout = 2000;
            }
            catch (Exception) { return -1; }

            // Set the connection state in order to call the GetModuleType method
            this.connectionState = 1;

            // Get the module type
            retval = this.GetModuleType(out this.systemType, out error);
            if (retval != 0)
            {
                this.connectionState = 0;
                return retval;
            }

            return 0;
        }

        /// <summary>
        /// Set the system has disconnected
        /// </summary>
        public void SetAsDisconnected()
        {
            this.connectionState = 0;
        }

        #region "Getter/Setter"

        /// <summary>
        /// Get the connection state of the MSX-E
        /// 0: not connected
        /// 1: connected
        /// </summary>
        public int ConnectionState
        {
            get { return this.connectionState; }
        }

        /// <summary>
        /// Get the IP address of the system
        /// </summary>
        public string IPAddress
        {
            get { return this.ip; }
        }

        /// <summary>
        /// Get the type of the system
        /// </summary>
        public string SystemType
        {
            get
            {
                if (this.connectionState != 1)
                    return string.Empty;
                else
                    return this.systemType;
            }
        }

        #endregion

        #region "SOAP API"

        /// <summary>
        /// Reads the type of the MSX-E system
        /// </summary>
        /// <param name="moduleType">Type of the MSX-E system</param>
        /// <param name="error">Error message set if the return value is -2 or -5</param>
        /// <returns>
        /// 0:  success
        /// -1: you must call the Connect function before
        /// -2: system error. Check the error variable for more informations
        /// -3: the method is not implemented on the system. Please check the type of your system
        /// -4: the system is not reachable (not present on the network, or no SOAP server on it)
        /// -5: other error. Check the error variable for more informations
        /// </returns>
        public int GetModuleType(out string moduleType, out string error)
        {
            // Default
            moduleType = string.Empty;
            error = string.Empty;

            // Check the connection state: Connect method should be called first
            if (this.connectionState != 1)
            {
                return -1;
            }

            // Call the SOAP function
            try
            {
                byte[] b = null;
                MXCommonWeb.DefaultResponse response = this.commonWeb.GetModuleType(out b);
                if (response.iReturnValue != 0)
                {
                    error = "The system returned an error when calling the MXCommon__GetModuleType function." + Environment.NewLine;
                    error += "Please check the documentation of this function for more information." + Environment.NewLine;
                    error += "(iReturnValue: " + response.iReturnValue.ToString() + ", syserrno: " + response.syserrno.ToString() + ")";
                    return -2;
                }

                // convert for the output
                moduleType = new ASCIIEncoding().GetString(b);
            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                return -3;
            }
            catch (System.Net.WebException)
            {
                return -4;
            }
            catch (Exception ex)
            {
                error = "Unhandled exception occured: " + ex.Message;
                return -5;
            }
            return 0;
        }

        #endregion
    }
}

