using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace InitialiseReadClear
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// Store the MSX-E object
        /// </summary>
        private Msxexxxx msxe;

        /// <summary>
        /// Possible resolutions
        /// </summary>
        private ulong[] resolution = {16, 32, 40, 64, 80, 100, 128, 160, 200, 256, 320, 400, 500, 512, 800, 1000, 1024, 1600, 2000, 2048, 4096, 8192};

        /// <summary>
        /// Constructor of the graphical interface
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Event handler: occurs once when the form is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // Load default parameters
            this.ipTextBox.Text = Properties.Settings.Default.ip;
            this.resolutionComboBox.SelectedIndex = 0;
        }

        /// <summary>
        /// Event handler: occurs when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Event handler: click on the "Initialise connection" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void initialiseConnectionButton_Click(object sender, EventArgs e)
        {
            // Check the validity of the IP address
            if (this.ipTextBox.Valid == false)
                return;

            // Update some graphics
            this.initialiseConnectionButton.Enabled = false;
            this.actionToolStripProgressBar.Visible = true;
            this.actionToolStripStatusLabel.Text = "Initialise the connection with the MSX-E system.";
            this.actionToolStripStatusLabel.Visible = true;
            this.sinusCosinusGroupBox.Enabled = false;

            // Start the connection background worker
            this.connectionBackgroundWorker.RunWorkerAsync(this.ipTextBox.Text);
        }

        /// <summary>
        /// Create the data source for the channel selection combo-box
        /// </summary>
        /// <param name="channelNb"></param>
        /// <returns></returns>
        private string[] GetChannelDataSource(ulong channelNb)
        {
            string[] src = new string[channelNb];
            for (ulong i = 0; i < channelNb; i++)
            {
                src[i] = i.ToString();
            }
            return src;
        }

        #region "Connection background worker"

        /// <summary>
        /// Job of the connection background worker
        /// The connection task is done in background since it may takes some time to initialise the system
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            int retval = 0;
            string error = string.Empty;

            // Construct the new MSX-E object
            Msxexxxx m = new Msxexxxx(this.ipTextBox.Text);

            // Do the connection
            retval = m.Connect(out error);
            e.Result = new ConnectionWorkerResult(m, retval, error);
        }

        /// <summary>
        /// Event handler: Connection background worker is completed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ConnectionWorkerResult r = (ConnectionWorkerResult)e.Result;

            // Error ?
            if (r.retval != 0)
            {
                string error = string.Empty;
                this.actionToolStripProgressBar.Visible = false;
                this.actionToolStripStatusLabel.Text = "Error occurred.";
                // Reset the system type
                this.systemInformationTextBox.Text = string.Empty;
                // Display the right text
                switch (r.retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + r.error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + r.error;
                        break;
                }
                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                // Store the object
                this.msxe = r.msxe;

                this.actionToolStripProgressBar.Visible = false;
                this.actionToolStripStatusLabel.Text = "Connection opened with the system: " + r.msxe.IPAddress;
                // Display the system type
                this.systemInformationTextBox.Text = r.msxe.SystemType;

                // Enable controls
                this.sinusCosinusGroupBox.Enabled = true;

                // Set data source
                this.channelComboBox.DataSource = this.GetChannelDataSource(r.msxe.SinCosChannelNb);
            }

            // Enable the "Initialise Connection" button
            this.initialiseConnectionButton.Enabled = true;
        }

        #endregion

        /// <summary>
        /// Event handler: click on the "Initialise Sinus/Cosinus" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void initialiseButton_Click(object sender, EventArgs e)
        {
            double signalPeriod = 0;
            string error = string.Empty;
            int retval = 0;
            ulong maximalFrequency = 0;

            // Reset maximal frequency
            this.maximalInputFrequencyTextBox.Text = string.Empty;

            if (this.channelComboBox.SelectedIndex < 0)
            {
                MessageBox.Show("Please select a channel.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (this.resolutionComboBox.SelectedIndex < 0)
            {
                MessageBox.Show("Please select a resolution.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            try
            {
                signalPeriod = Convert.ToDouble(this.signalPeriodTextBox.Text);
            }
            catch (Exception)
            {
                MessageBox.Show("Please give a valid value for the parameter signal period.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // Call SOAP function
            retval = this.msxe.SinCosInit((ulong)this.channelComboBox.SelectedIndex,
                signalPeriod,
                this.resolution[this.resolutionComboBox.SelectedIndex],
                out maximalFrequency,
                out error);
            if (retval != 0)
            {
                switch (retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + error;
                        break;
                }
                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // Set maximal frequency information
            this.maximalInputFrequencyTextBox.Text = maximalFrequency.ToString();

            // Refresh configuration
            this.RefreshCurrentConfiguration();
        }

        /// <summary>
        /// Event handler: click on the "Release Sinus/Cosinus" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void releaseButton_Click(object sender, EventArgs e)
        {
            string error = string.Empty;
            int retval = 0;

            if (this.channelComboBox.SelectedIndex < 0)
            {
                MessageBox.Show("Please select a channel.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // Call SOAP function
            retval = this.msxe.SinCosRelease((ulong)this.channelComboBox.SelectedIndex,
                out error);
            if (retval != 0)
            {
                switch (retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + error;
                        break;
                }
                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // Refresh current configuration
            this.RefreshCurrentConfiguration();
        }

        /// <summary>
        /// Event handler: click on the "Read Sinus/Cosinus" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void readValueButton_Click(object sender, EventArgs e)
        {
            string error = string.Empty;
            int retval = 0;
            double value = 0;
            ulong rawvalue = 0;
            ulong measureerror = 0;

            if (this.channelComboBox.SelectedIndex < 0)
            {
                MessageBox.Show("Please select a channel.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // Call SOAP function
            retval = this.msxe.SinCosRead((ulong)this.channelComboBox.SelectedIndex,
                out value,
                out rawvalue,
                out measureerror,
                out error);
            if (retval != 0)
            {
                switch (retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + error;
                        break;
                }
                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // Set the measured value
            this.valueRawTextBox.Text = rawvalue.ToString();
            this.valueMmTextBox.Text = value.ToString("F5");
            this.measureErrorTextBox.Text = measureerror.ToString();
        }

        /// <summary>
        /// Event handler: click on the "Clear Sinus/Cosinus" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void clearButton_Click(object sender, EventArgs e)
        {
            string error = string.Empty;
            int retval = 0;

            if (this.channelComboBox.SelectedIndex < 0)
            {
                MessageBox.Show("Please select a channel.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // Call SOAP function
            retval = this.msxe.SinCosClear((ulong)this.channelComboBox.SelectedIndex,
                out error);
            if (retval != 0)
            {
                switch (retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + error;
                        break;
                }
                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
        }

        /// <summary>
        /// Refresh the current configuration
        /// </summary>
        private void RefreshCurrentConfiguration()
        {
            if (this.channelComboBox.SelectedIndex < 0)
            {
                return;
            }

            string error = string.Empty;
            int retval = 0;
            ulong initialisationState = 0;
            double signalPeriod = 0;
            ulong resolution = 0;
            ulong indexState = 0;
            ulong indexEdge = 0;
            ulong indexAction = 0;
            ulong compareLogicState = 0;
            double compareLogicValue = 0;
            ulong compareLogicValueFormat = 0;
            ulong compareLogicMode = 0;
            ulong compareLogicSynchro = 0;

            // Get the current configuration of the selected channel
            retval = this.msxe.SinCosGetConfiguration((ulong)channelComboBox.SelectedIndex,
                out initialisationState,
                out signalPeriod,
                out resolution,
                out indexState, out indexEdge, out indexAction,
                out compareLogicState, out compareLogicValue, out compareLogicValueFormat,
                out compareLogicMode, out compareLogicSynchro,
                out error);
            if (retval != 0)
            {
                switch (retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + error;
                        break;
                }
                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // Update graphic
            this.resolutionTextBox.Text = resolution.ToString();
            this.currentSignalPeriodTextBox.Text = signalPeriod.ToString("F5");
            if (initialisationState == 1)
            {
                this.initialisationStateTextBox.Text = "1 (Initialised)";
                this.initialisationStateTextBox.BackColor = Color.LightGreen;
            }
            else
            {
                this.initialisationStateTextBox.Text = "0 (Uninitialised)";
                this.initialisationStateTextBox.BackColor = Color.Coral;
            }
            if (indexState == 1)
            {
                this.indexStateTextBox.Text = "1 (Initialised)";
                this.indexStateTextBox.BackColor = Color.LightGreen;
            }
            else
            {
                this.indexStateTextBox.Text = "0 (Uninitialised)";
                this.indexStateTextBox.BackColor = Color.Coral;
            }

            if (indexEdge > 0)
            {
                this.indexEdgeComboBox.SelectedIndex = (int)(indexEdge - 1);
            }
            else
            {
                this.indexEdgeComboBox.SelectedIndex = 0;
            }
            this.indexClearCheckBox.Checked = ((indexAction & 1) != 0);
            this.indexSynchroCheckBox.Checked = ((indexAction & 2) != 0);
            if (compareLogicState == 1)
            {
                this.compareLogicStateTextBox.Text = "1 (Initialised)";
                this.compareLogicStateTextBox.BackColor = Color.LightGreen;
            }
            else
            {
                this.compareLogicStateTextBox.Text = "0 (Uninitialised)";
                this.compareLogicStateTextBox.BackColor = Color.Coral;
            }
            this.compareLogicValueTextBox.Text = compareLogicValue.ToString();
            this.compareLogicValueFormatComboBox.SelectedIndex = (int)compareLogicValueFormat;
            this.compareLogicModeComboBox.SelectedIndex = (int)compareLogicMode;
            this.compareLogicSynchroComboBox.SelectedIndex = (int)compareLogicSynchro;
            this.currentStateGroupBox.Text = "Current status of channel " + channelComboBox.SelectedIndex.ToString();
        }

        /// <summary>
        /// Event handler: change the selected channel
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void channelComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.RefreshCurrentConfiguration();
        }

        /// <summary>
        /// Event handler: click on the refresh button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void getCurrentConfigurationButton_Click(object sender, EventArgs e)
        {
            this.RefreshCurrentConfiguration();
        }

        /// <summary>
        /// Event handler: click on the "Clear Sinus/Cosinus" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void clearErrorButton_Click(object sender, EventArgs e)
        {
            string error = string.Empty;
            int retval = 0;

            if (this.channelComboBox.SelectedIndex < 0)
            {
                MessageBox.Show("Please select a channel.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // Call SOAP function
            retval = this.msxe.SinCosClearError((ulong)this.channelComboBox.SelectedIndex,
                out error);
            if (retval != 0)
            {
                switch (retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + error;
                        break;
                }
                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
        }
    }
}